@@ -1,5 +1,5 @@
 import datetime
-from minty.entity import Entity
+from minty.entity import Entity, EntityCollection
 from pyramid.renderers import JSON
 from uuid import UUID
 
@@ -7,6 +7,7 @@ from uuid import UUID
 class jsonapi(JSON):
     jsonapi_adapters = {
         Entity: "_entity_adapter",
+        EntityCollection: "_entity_collection_adapter",
         UUID: "_uuid_adapter",
         datetime.datetime: "_datetime_adapter",
     }
@@ -17,13 +18,11 @@ class jsonapi(JSON):
         This will add some default adapters to the system, like UUID, entity
         parsing and datetime handling
         """
-        rv = super().__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         for obj, adapter in self.jsonapi_adapters.items():
             self.add_adapter(obj, getattr(self, adapter))
 
-        return rv
-
     def __call__(self, info):
         """ Returns a plain JSON-encoded string with content-type
         ``application/json``. The content-type may be overridden by
@@ -46,6 +45,16 @@ class jsonapi(JSON):
 
         return _render
 
+    def _entity_collection_adapter(self, obj, request):
+        return [entity for entity in obj]
+
+    def __links_for_entity(self, entity):
+        self_link = self.view.create_link_from_entity(entity)
+        if self_link:
+            return {"links": {"self": self_link}}
+        else:
+            return {}
+
     def _entity_adapter(self, obj, request):
         """ Transforms an entity in a JSON:API styled format
 
@@ -62,14 +71,19 @@ class jsonapi(JSON):
             if relationship is not None:
                 relationshipdata = relationship.dict()
 
+                metadata = {}
+                if relationshipdata["entity_meta_summary"] is not None:
+                    metadata["meta"] = {
+                        "summary": relationshipdata["entity_meta_summary"]
+                    }
+
                 object_relationships[relationshipkey] = {
                     "data": {
                         "type": relationshipdata["entity_type"],
                         "id": relationshipdata["entity_id"],
                     },
-                    "links": {
-                        "self": self.view.create_link_from_entity(relationship)
-                    },
+                    **metadata,
+                    **self.__links_for_entity(relationship),
                 }
 
             del object_attributes[relationshipkey]
@@ -77,15 +91,17 @@ class jsonapi(JSON):
         for entity_id__field in obj.entity_id__fields:
             del object_attributes[entity_id__field]
 
+        relationships_block = {}
+        if object_relationships != {}:
+            relationships_block["relationships"] = object_relationships
+
         return {
-            "data": {
-                "type": obj.entity_type,
-                "id": obj.entity_id,
-                "meta": {"summary": obj.entity_meta_summary},
-                "attributes": object_attributes,
-                "relationships": object_relationships,
-            },
-            "links": {"self": self.view.create_link_from_entity(obj)},
+            "type": obj.entity_type,
+            "id": obj.entity_id,
+            "meta": {"summary": obj.entity_meta_summary},
+            "attributes": object_attributes,
+            **relationships_block,
+            **self.__links_for_entity(obj),
         }
 
     def _uuid_adapter(self, obj, request):
